"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const protractor_element_extend_1 = require("protractor-element-extend");
class ArrayFragment extends protractor_element_extend_1.BaseArrayFragment {
    filter(filterFn) {
        return super.filter(this.makeFnSequential(filterFn));
    }
    find(filterFn) {
        return __awaiter(this, void 0, void 0, function* () {
            const count = yield this.count();
            if (count === 0) {
                throw new Error('Wow, you try to apply find() on empty Array');
            }
            for (let i = 0; i < count; i++) {
                const fragment = yield this.get(i);
                try {
                    if (yield filterFn(fragment, i)) {
                        return fragment;
                    }
                }
                catch (e) {
                    throw new Error(`Hello my friend, you get an error in filter function: "${e}"`);
                }
            }
            throw new Error('Find function did not match any fragment');
        });
    }
    /**
     *
     * Represents the ArrayFragment as an array of Fragments.
     *
     * @returns {Array<Fragment>} Returns a promise, which resolves
     * to a list of Fragments.
     */
    asFragments() {
        return __awaiter(this, void 0, void 0, function* () {
            const fragments = [];
            yield this.each((fragment) => fragments.push(fragment));
            return fragments;
        });
    }
    map(mapFn) {
        return super.map(this.makeFnSequential(mapFn));
    }
    reduce(reduceFn, initialValue) {
        return super.reduce(this.makeFnSequential(reduceFn), initialValue);
    }
    each(fn) {
        return super.map(this.makeFnSequential(fn)).then(() => {
            return null;
        });
    }
    makeFnSequential(func) {
        let promise = Promise.resolve();
        return (...args) => {
            promise = promise.then(() => {
                return func(...args);
            });
            return promise;
        };
    }
}
exports.ArrayFragment = ArrayFragment;
//# sourceMappingURL=array-fragment.wrapper.js.map