"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const wait_conditions_1 = require("../waiters/wait-conditions");
const protractor_element_extend_1 = require("protractor-element-extend");
const protractor_1 = require("protractor");
const oi_verify_1 = require("oi-verify");
class Fragment extends protractor_element_extend_1.BaseFragment {
    getFragmentText() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.waitForVisible();
            }
            catch (e) {
                throw new Error(`Can not get fragment text of not visible fragment. ${e}`);
            }
            return yield this.getText();
        });
    }
    clickOn() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.waitForClickable();
            }
            catch (e) {
                throw new Error(`Can not click on fragment of not clickable fragment. ${e}`);
            }
            yield this.click();
        });
    }
    getDataTestId() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getFragmentAttribute('data-test-id');
        });
    }
    pressEscape() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.waitForClickable();
            }
            catch (e) {
                throw new Error(`Can not press escape of not clickable fragment. ${e}`);
            }
            yield this.sendKeys(protractor_1.protractor.Key.ESCAPE);
        });
    }
    type(text) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.clearInput();
            yield this.sendKeys(text);
        });
    }
    clearInput() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.waitForClickable();
            }
            catch (e) {
                throw new Error(`Can not clear the input of not clickable fragment. ${e}`);
            }
            yield this.clear();
        });
    }
    getFragmentAttribute(name) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.waitForVisible();
            }
            catch (e) {
                throw new Error(`Can not get attribute ${name} of not visible fragment. ${e}`);
            }
            const attribute = yield this.getAttribute(name);
            if (attribute === null) {
                throw new Error(`Watch out, fragment does not have ${name} attribute!`);
            }
            return attribute;
        });
    }
    waitForVisible() {
        return __awaiter(this, void 0, void 0, function* () {
            yield wait_conditions_1.WaitConditions.visible(this);
        });
    }
    waitForNotVisible() {
        return __awaiter(this, void 0, void 0, function* () {
            yield wait_conditions_1.WaitConditions.invisible(this);
        });
    }
    waitForClickable() {
        return __awaiter(this, void 0, void 0, function* () {
            yield wait_conditions_1.WaitConditions.clickable(this);
        });
    }
    /**
     * Double-clicks a mouse button.
     *
     * DoubleClick() from ActionSequence class doesn't work properly
     * on Chrome
     *
     */
    doubleClick() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.clickOn();
            yield this.clickOn();
        });
    }
    isClassAttributeIncludes(substring) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.getFragmentAttribute('class')).includes(substring);
        });
    }
    expectToContainText(substring) {
        return __awaiter(this, void 0, void 0, function* () {
            yield oi_verify_1.Verify.textContains(() => this.getFragmentText(), substring);
        });
    }
    expectToNotContainText(substring) {
        return __awaiter(this, void 0, void 0, function* () {
            yield oi_verify_1.Verify.textNotContains(() => this.getFragmentText(), substring);
        });
    }
    expectClassToContain(substring) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.expectToContainTextInAttribute('class', substring);
        });
    }
    expectClassToNotContain(substring) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.expectToNotContainTextInAttribute('class', substring);
        });
    }
    expectToContainTextInAttribute(attribute, substring) {
        return __awaiter(this, void 0, void 0, function* () {
            yield oi_verify_1.Verify.textContains(() => this.getFragmentAttribute(attribute), substring);
        });
    }
    expectToNotContainTextInAttribute(attribute, substring) {
        return __awaiter(this, void 0, void 0, function* () {
            yield oi_verify_1.Verify.textNotContains(() => this.getFragmentAttribute(attribute), substring);
        });
    }
}
exports.Fragment = Fragment;
//# sourceMappingURL=fragment.wrapper.js.map